/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GPELClientTestCase.java,v 1.40 2007/07/19 00:35:09 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.test;

import java.io.IOException;
import java.net.URI;
import java.security.GeneralSecurityException;
import java.util.Date;
import java.util.List;

import javax.xml.namespace.QName;

import junit.framework.TestSuite;

import org.globus.gsi.CertUtil;
import org.gpel.client.GcInstance;
import org.gpel.client.GcSearchList;
import org.gpel.client.GcSearchResult;
import org.gpel.client.security.GpelUserX509Credential;
import org.ietf.jgss.GSSCredential;
import org.xmlpull.infoset.XmlElement;

import xsul.lead.LeadContextHeader;
import xsul.wsif.WSIFMessage;
import xsul5.MLogger;
import xsul5.wsdl.WsdlDefinitions;
import edu.indiana.extreme.xbaya.XBayaException;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistryException;
import edu.indiana.extreme.xbaya.component.ws.WSComponentPort;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.gpel.GPELClient.WorkflowType;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.lead.LEADWorkflowInvoker;
import edu.indiana.extreme.xbaya.lead.LeadContextHeaderHelper;
import edu.indiana.extreme.xbaya.test.util.WorkflowCreator;
import edu.indiana.extreme.xbaya.util.XMLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class GPELClientTestCase extends XBayaTestCase {

    private static final MLogger logger = MLogger.getLogger();

    /**
     * @param args
     */
    public static void main(String[] args) {
        junit.textui.TestRunner.run(new TestSuite(GPELClientTestCase.class));
    }

    /**
     * @throws ComponentException
     * @throws GraphException
     * @throws GPELException
     * @throws ComponentRegistryException
     */
    public void XtestDeploy() throws ComponentException, GraphException,
            GPELException, ComponentRegistryException {
        WorkflowCreator creator = new WorkflowCreator();
        Workflow workflow = creator.createComplexMathWorkflow();
        workflow.setName("this is test name");
        workflow.setDescription("this is test description");
        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());

        client.createScriptAndDeploy(workflow, false);

        GcSearchList list = client.list();
        for (GcSearchResult result : list.results()) {
            logger.info(result.getTitle());
        }

        client.load(workflow.getGPELTemplateID());
    }

    /**
     * @throws ComponentException
     * @throws GraphException
     * @throws GPELException
     * @throws ComponentRegistryException
     */
    public void XtestRedeploy() throws ComponentException, GraphException,
            GPELException, ComponentRegistryException {
        WorkflowCreator creator = new WorkflowCreator();
        Workflow workflow = creator.createComplexMathWorkflow();
        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());

        client.createScriptAndDeploy(workflow, false);

        client.createScriptAndDeploy(workflow, true);
    }

    /**
     * @throws GPELException
     */
    public void XtestListWorkflowInstances() throws GPELException {
        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());
        GcSearchList searchList = client.list(100, WorkflowType.INSTANCE);
        for (GcSearchResult result : searchList.results()) {
            URI id = result.getId();
            logger.info("id: " + id);
        }
    }

    /**
     * @throws GPELException
     */
    public void XtestList() throws GPELException {

        // The URL of the GPEL Engine.
        URI gpelEngineURL = this.configuration.getGPELEngineURL();

        // Create a client.
        GPELClient client = new GPELClient();
        client.setEngineURL(gpelEngineURL);

        // XXX This is supposed to return the workflow templates related to the
        // user only. But users are not implemented in GPEL yet.
        GcSearchList results = client.list();

        for (GcSearchResult result : results.results()) {

            // The title to show to the user.
            String title = result.getTitle();
            logger.info("title: " + title);

            // ID to load the workflow from the GPEL engine.
            URI templateID = result.getId();
            logger.info("templateID: " + templateID);

            // XXX Not implemented yet? It always returns -1.
            Date lastModified = new Date(result.getLastModfied());
            logger.info("lastModified: " + lastModified);
        }
    }

    /**
     * @throws ComponentException
     * @throws GraphException
     * @throws GPELException
     * @throws ComponentRegistryException
     */
    public void XtestStart() throws ComponentException, GraphException,
            GPELException, ComponentRegistryException {
        WorkflowCreator creator = new WorkflowCreator();
        Workflow workflow = creator.createSimpleMathWorkflow();

        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());

        client.createScriptAndDeploy(workflow, false);

        // Instantiate the workflow template.
        GcInstance instance = client.instantiate(workflow, this.configuration
                .getDSCURL());

        // Start the workflow instance.
        WsdlDefinitions wsdl = client.start(instance);

        logger.info(wsdl.xmlStringPretty());
    }

    /**
     * @throws XBayaException
     */
    public void testLoadInstance() throws XBayaException {
        WorkflowCreator creator = new WorkflowCreator();
        Workflow workflow0 = creator.createComplexMathWorkflow();

        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());

        // Deploy
        URI templateID = client.createScriptAndDeploy(workflow0, false);

        // Load a workflow template from the GPEL Engine.
        Workflow workflowTemplate = client.load(templateID);

        // Instantiate the workflow template.
        GcInstance instance = client.instantiate(workflowTemplate,
                this.configuration.getDSCURL());
        // ID to retrieve the workflow instance
        URI instanceID = instance.getInstanceId();
        logger.info("instanceID: " + instanceID);

        // Start the workflow instance.
        // WsdlDefinitions wsdl = client.start(instance);

        Workflow workflowInstance = client.load(instanceID,
                WorkflowType.INSTANCE);
        assertNotNull(workflowInstance);
    }

    /**
     * @throws XBayaException
     */
    public void XtestInvoke() throws XBayaException {

        WorkflowCreator creator = new WorkflowCreator();
        Workflow workflow0 = creator.createGFacWorkflow();

        GPELClient client = new GPELClient();
        client.setEngineURL(this.configuration.getGPELEngineURL());

        URI templateID = client.createScriptAndDeploy(workflow0, false);

        // Load a workflow template from the GPEL Engine.
        Workflow workflow = client.load(templateID);

        // Get the metadata for input.
        XmlElement inputAppinfo = workflow.getInputMetadata();
        logger
                .info("inputAppinfo: "
                        + XMLUtil.xmlElementToString(inputAppinfo));

        // Get the input information
        List<WSComponentPort> inputs = workflow.getInputs();

        for (WSComponentPort input : inputs) {
            // Show the information of each input.

            // Name
            String name = input.getName();
            logger.info("name: " + name);

            // Type
            QName type = input.getType();
            logger.info("type: " + type);

            // Metadata as XML
            XmlElement appinfo = input.getAppinfo();
            logger.info("appinfo: " + XMLUtil.xmlElementToString(appinfo));
            if (appinfo != null) {
                // Parse the simple case.
                for (XmlElement element : appinfo.requiredElementContent()) {
                    String tag = element.getName();
                    String value = element.requiredText();
                    logger.info(tag + " = " + value);
                }
            }

            // Set a value to each input.
            input.setValue("200");
        }

        // Instantiate the workflow template.
        GcInstance instance = client.instantiate(workflow, this.configuration
                .getDSCURL());
        // ID to retrieve the workflow instance
        URI instanceID = instance.getInstanceId();
        logger.info("instanceID: " + instanceID);

        // Start the workflow instance.
        WsdlDefinitions wsdl = client.start(instance);

        // Create lead context.
        LeadContextHeaderHelper leadContextHelper = new LeadContextHeaderHelper();
        leadContextHelper.setXBayaConfiguration(this.configuration);
        LeadContextHeader leadContext = leadContextHelper
                .getLeadContextHeader();

        URI messageBoxURL = null;
        if (this.configuration.isPullMode()) {
            messageBoxURL = this.configuration.getMessageBoxURL();
        }

        // Create an invoker to invoke the workflow instance.
        LEADWorkflowInvoker invoker = new LEADWorkflowInvoker(wsdl,
                leadContext, messageBoxURL);

        // Set the input values to the invoker.
        invoker.setInputs(inputs);

        // Invoke the workflow. This will block, so you may want to do it in a
        // thread.
        boolean success = invoker.invoke();
        logger.info("success: " + success);

        // We don't need to wait for the outputs.

        if (success) {
            List<WSComponentPort> outputs = invoker.getOutputs();
            for (WSComponentPort output : outputs) {
                String name = output.getName();
                logger.info("name: " + name);
                Object value = output.getValue();
                logger.info("value: " + value);
            }
        } else {
            WSIFMessage fault = invoker.getFault();
            logger.info("fault: " + fault);
        }

    }

    /**
     * @throws GPELException
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void XtestSecurity() throws GPELException, IOException,
            GeneralSecurityException {
        boolean userCred = true;

        URI engineURL = this.configuration.getGPELEngineURL();
        GSSCredential proxy = null;

        String trustedcerts = System.getProperty("trustedcerts");
        String certskey = System.getProperty("certskey");

        GpelUserX509Credential credential;
        if (userCred) {
            // Using user credential
            credential = new GpelUserX509Credential(proxy, CertUtil
                    .loadCertificates(trustedcerts));
        } else {
            // Using service certificate.
            credential = new GpelUserX509Credential(certskey, trustedcerts);
        }
        GPELClient client = new GPELClient(engineURL, credential);
        assertNotNull(client);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
